@@ -2,21 +2,15 @@
 //
 // Audio processing part of this file ported from ffmpeg/libavfilter/af_loudnorm.c
 //
-// Copyright (c) 2016 Kyle Swanson <k@ylo.ph>
+// Original C code
+//   Copyright (c) 2016 Kyle Swanson <k@ylo.ph>
+// licensed under the LGPL-2.1+ and generously relicensed to MPL-2.0.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// FFmpeg is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public
-// License along with FFmpeg; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -25,7 +19,7 @@ use gst::{gst_debug, gst_error, gst_info, gst_log};
 
 use std::mem;
 use std::sync::Mutex;
-use std::{i32, u64};
+use std::u64;
 
 use byte_slice_cast::*;
 
@@ -235,7 +229,9 @@ impl State {
             let distance_ts = distance_samples
                 .mul_div_floor(*gst::ClockTime::SECOND, self.info.rate() as u64)
                 .map(gst::ClockTime::from_nseconds);
-            let pts = pts.zip(distance_ts).map(|(pts, dist)| pts + dist);
+            let pts = pts
+                .opt_checked_add(distance_ts)
+                .map_err(|_| gst::FlowError::Error)?;
 
             let inbuf = self
                 .adapter
@@ -276,7 +272,9 @@ impl State {
         let distance_ts = distance_samples
             .mul_div_floor(*gst::ClockTime::SECOND, self.info.rate() as u64)
             .map(gst::ClockTime::from_nseconds);
-        let pts = pts.zip(distance_ts).map(|(pts, dist)| pts + dist);
+        let pts = pts
+            .opt_checked_add(distance_ts)
+            .map_err(|_| gst::FlowError::Error)?;
 
         let mut _mapped_inbuf = None;
         let src = if self.adapter.available() > 0 {
@@ -1028,7 +1026,7 @@ impl State {
                     // Calculate at which point we would reach the new gain reduction
                     // relative to 0.0 == attack window start, 1.0 attack window end.
                     let new_end = (gain_reduction - self.gain_reduction[0]) / old_slope;
-                    assert!(new_end >= 1.0);
+                    let new_end = f64::max(new_end, 1.0);
 
                     // New start of the window, this will be in the past
                     let new_start = new_end - 1.0;
@@ -1049,7 +1047,7 @@ impl State {
 
                     // Calculate the current position in the attack window
                     let cur_pos = (current_gain_reduction - self.gain_reduction[0]) / old_slope;
-                    assert!(cur_pos >= 0.0 && cur_pos <= 1.0);
+                    let cur_pos = f64::clamp(cur_pos, 0.0, 1.0);
                     self.env_cnt = ((LIMITER_ATTACK_WINDOW as f64 - 1.0) * cur_pos) as usize;
 
                     // Need to sustain in any case for this many samples to actually
@@ -1684,6 +1682,7 @@ impl AudioLoudNorm {
         pad.event_default(Some(element), event)
     }
 
+    #[allow(clippy::single_match)]
     fn src_query(
         &self,
         pad: &gst::Pad,
@@ -1701,7 +1700,7 @@ impl AudioLoudNorm {
                     q.set(
                         live,
                         min_latency + 3 * gst::ClockTime::SECOND,
-                        max_latency.map(|max| max + 3 * gst::ClockTime::SECOND),
+                        max_latency.opt_add(3 * gst::ClockTime::SECOND),
                     );
                     true
                 } else {
@@ -1764,7 +1763,7 @@ impl ObjectImpl for AudioLoudNorm {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_double(
+                glib::ParamSpecDouble::new(
                     "loudness-target",
                     "Loudness Target",
                     "Loudness target in LUFS",
@@ -1773,7 +1772,7 @@ impl ObjectImpl for AudioLoudNorm {
                     DEFAULT_LOUDNESS_TARGET,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_double(
+                glib::ParamSpecDouble::new(
                     "loudness-range-target",
                     "Loudness Range Target",
                     "Loudness range target in LU",
@@ -1782,7 +1781,7 @@ impl ObjectImpl for AudioLoudNorm {
                     DEFAULT_LOUDNESS_RANGE_TARGET,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_double(
+                glib::ParamSpecDouble::new(
                     "max-true-peak",
                     "Maximum True Peak",
                     "Maximum True Peak in dbTP",
@@ -1791,7 +1790,7 @@ impl ObjectImpl for AudioLoudNorm {
                     DEFAULT_MAX_TRUE_PEAK,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_double(
+                glib::ParamSpecDouble::new(
                     "offset",
                     "Offset Gain",
                     "Offset Gain in LU",
@@ -1864,6 +1863,8 @@ impl ObjectImpl for AudioLoudNorm {
     }
 }
 
+impl GstObjectImpl for AudioLoudNorm {}
+
 impl ElementImpl for AudioLoudNorm {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -1880,15 +1881,12 @@ impl ElementImpl for AudioLoudNorm {
 
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
-            let caps = gst::Caps::new_simple(
-                "audio/x-raw",
-                &[
-                    ("format", &gst_audio::AUDIO_FORMAT_F64.to_str()),
-                    ("rate", &192_000i32),
-                    ("channels", &gst::IntRange::<i32>::new(1, std::i32::MAX)),
-                    ("layout", &"interleaved"),
-                ],
-            );
+            let caps = gst::Caps::builder("audio/x-raw")
+                .field("format", gst_audio::AUDIO_FORMAT_F64.to_str())
+                .field("rate", 192_000i32)
+                .field("channels", gst::IntRange::new(1, std::i32::MAX))
+                .field("layout", "interleaved")
+                .build();
             let src_pad_template = gst::PadTemplate::new(
                 "src",
                 gst::PadDirection::Src,
@@ -1911,6 +1909,7 @@ impl ElementImpl for AudioLoudNorm {
         PAD_TEMPLATES.as_ref()
     }
 
+    #[allow(clippy::single_match)]
     fn change_state(
         &self,
         element: &Self::Type,
