@@ -1,10 +1,10 @@
 // Copyright (C) 2021 Sebastian Dröge <sebastian@centricular.com>
 //
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
+//
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -33,22 +33,22 @@ static CAT: Lazy<gst::DebugCategory> = Lazy::new(|| {
     )
 });
 
-#[glib::gflags("EbuR128LevelMode")]
+#[glib::flags(name = "EbuR128LevelMode")]
 enum Mode {
-    #[gflags(name = "Calculate momentary loudness (400ms)", nick = "momentary")]
+    #[flags_value(name = "Calculate momentary loudness (400ms)", nick = "momentary")]
     MOMENTARY = 0b00000001,
-    #[gflags(name = "Calculate short-term loudness (3s)", nick = "short-term")]
+    #[flags_value(name = "Calculate short-term loudness (3s)", nick = "short-term")]
     SHORT_TERM = 0b00000010,
-    #[gflags(
+    #[flags_value(
         name = "Calculate relative threshold and global loudness",
         nick = "global"
     )]
     GLOBAL = 0b00000100,
-    #[gflags(name = "Calculate loudness range", nick = "loudness-range")]
+    #[flags_value(name = "Calculate loudness range", nick = "loudness-range")]
     LOUDNESS_RANGE = 0b00001000,
-    #[gflags(name = "Calculate sample peak", nick = "sample-peak")]
+    #[flags_value(name = "Calculate sample peak", nick = "sample-peak")]
     SAMPLE_PEAK = 0b00010000,
-    #[gflags(name = "Calculate true peak", nick = "true-peak")]
+    #[flags_value(name = "Calculate true peak", nick = "true-peak")]
     TRUE_PEAK = 0b00100000,
 }
 
@@ -147,7 +147,7 @@ impl ObjectImpl for EbuR128Level {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_flags(
+                glib::ParamSpecFlags::new(
                     "mode",
                     "Mode",
                     "Selection of metrics to calculate",
@@ -155,14 +155,14 @@ impl ObjectImpl for EbuR128Level {
                     DEFAULT_MODE.bits() as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "post-messages",
                     "Post Messages",
                     "Whether to post messages on the bus for each interval",
                     DEFAULT_POST_MESSAGES,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "interval",
                     "Interval",
                     "Interval in nanoseconds for posting messages",
@@ -235,6 +235,8 @@ impl ObjectImpl for EbuR128Level {
     }
 }
 
+impl GstObjectImpl for EbuR128Level {}
+
 impl ElementImpl for EbuR128Level {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -254,21 +256,18 @@ impl ElementImpl for EbuR128Level {
             let caps = gst::Caps::builder("audio/x-raw")
                 .field(
                     "format",
-                    &gst::List::new(&[
-                        &gst_audio::AUDIO_FORMAT_S16.to_str(),
-                        &gst_audio::AUDIO_FORMAT_S32.to_str(),
-                        &gst_audio::AUDIO_FORMAT_F32.to_str(),
-                        &gst_audio::AUDIO_FORMAT_F64.to_str(),
+                    gst::List::new([
+                        gst_audio::AUDIO_FORMAT_S16.to_str(),
+                        gst_audio::AUDIO_FORMAT_S32.to_str(),
+                        gst_audio::AUDIO_FORMAT_F32.to_str(),
+                        gst_audio::AUDIO_FORMAT_F64.to_str(),
                     ]),
                 )
-                .field(
-                    "layout",
-                    &gst::List::new(&[&"interleaved", &"non-interleaved"]),
-                )
+                .field("layout", gst::List::new(["interleaved", "non-interleaved"]))
                 // Limit from ebur128
-                .field("rate", &gst::IntRange::<i32>::new(1, 2_822_400))
+                .field("rate", gst::IntRange::new(1i32, 2_822_400))
                 // Limit from ebur128
-                .field("channels", &gst::IntRange::<i32>::new(1, 64))
+                .field("channels", gst::IntRange::new(1i32, 64))
                 .build();
             let src_pad_template = gst::PadTemplate::new(
                 "src",
@@ -481,12 +480,12 @@ impl BaseTransformImpl for EbuR128Level {
 
             // The timestamp we report in messages is always the timestamp until which measurements
             // are included, not the starting timestamp.
-            timestamp = timestamp.map(|ts| {
-                ts + to_process
+            timestamp = timestamp.opt_add(
+                to_process
                     .mul_div_floor(*gst::ClockTime::SECOND, state.info.rate() as u64)
                     .map(gst::ClockTime::from_nseconds)
-                    .unwrap()
-            });
+                    .unwrap(),
+            );
 
             // Post a message whenever an interval is full
             if state.interval_frames_remaining.is_zero() {
@@ -497,9 +496,9 @@ impl BaseTransformImpl for EbuR128Level {
                     let stream_time = segment.as_ref().and_then(|s| s.to_stream_time(timestamp));
 
                     let mut s = gst::Structure::builder("ebur128-level")
-                        .field("timestamp", &timestamp)
-                        .field("running-time", &running_time)
-                        .field("stream-time", &stream_time)
+                        .field("timestamp", timestamp)
+                        .field("running-time", running_time)
+                        .field("stream-time", stream_time)
                         .build();
 
                     if state.ebur128.mode().contains(ebur128::Mode::M) {
@@ -566,7 +565,7 @@ impl BaseTransformImpl for EbuR128Level {
                             .collect::<Result<Vec<_>, _>>();
 
                         match peaks {
-                            Ok(peaks) => s.set("sample-peak", &gst::Array::from_owned(peaks)),
+                            Ok(peaks) => s.set("sample-peak", gst::Array::from(peaks)),
                             Err(err) => {
                                 gst_error!(CAT, obj: element, "Failed to get sample peaks: {}", err)
                             }
@@ -579,7 +578,7 @@ impl BaseTransformImpl for EbuR128Level {
                             .collect::<Result<Vec<_>, _>>();
 
                         match peaks {
-                            Ok(peaks) => s.set("true-peak", &gst::Array::from_owned(peaks)),
+                            Ok(peaks) => s.set("true-peak", gst::Array::from(peaks)),
                             Err(err) => {
                                 gst_error!(CAT, obj: element, "Failed to get true peaks: {}", err)
                             }
